import { EMPTY_STRING } from '@alfa-client/tech-shared';
import {
  BenutzerE2EComponent,
  BenutzerListE2EComponent,
  BenutzerListItemE2EComponent,
} from '../../components/benutzer/benutzer.e2e.component';
import { E2EBenutzerValidationMessages } from '../../model/benutzer.messages';
import { AdminUserE2E } from '../../model/util';
import { contains, exist, haveText, notBeEnabled, notContains, notExist } from '../../support/cypress.util';
import { AlfaRollen } from '../../support/user-util';

export class E2EBenutzerVerifier {
  private noOrganisationsEinheitAssignedText: string = 'keine zuständige Stelle zugewiesen';

  private benutzerListPage: BenutzerListE2EComponent = new BenutzerListE2EComponent();
  private benutzerPage: BenutzerE2EComponent = new BenutzerE2EComponent();

  public verfiyOrganisationsEinheitInBenutzerList(organisationsEinheit: string, userName: string): void {
    const benutzerItem: BenutzerListItemE2EComponent = this.getBenutzerItem(userName);
    contains(benutzerItem.getOrganisationsEinheiten(), organisationsEinheit);
  }

  public verifyNoOrganisationsEinheitInBenutzerList(userName: string): void {
    const benutzerItem: BenutzerListItemE2EComponent = this.getBenutzerItem(userName);
    contains(benutzerItem.getNoOrganisationsEinheitText(), this.noOrganisationsEinheitAssignedText);
  }

  public verifyOrganisationsEinheitenInFormular(organisationsEinheiten: string[]): void {
    organisationsEinheiten.forEach((organisationsEinheit) => {
      exist(this.benutzerPage.getOrganisationsEinheitCheckbox(organisationsEinheit));
    });
  }

  public verifyUserVorname(name: string): void {
    this.benutzerPage.getVornameInput().should('have.value', name);
  }

  public verifyAlfaRolesCorrectlyChecked(user: AdminUserE2E): void {
    if (user.isUser) {
      notBeEnabled(this.benutzerPage.getLoeschenCheckbox().getRoot());
      notBeEnabled(this.benutzerPage.getPostCheckbox().getRoot());
    }
    if (user.isLoeschen) {
      notBeEnabled(this.benutzerPage.getUserCheckbox().getRoot());
      notBeEnabled(this.benutzerPage.getPostCheckbox().getRoot());
    }
    if (user.isPoststelle) {
      notBeEnabled(this.benutzerPage.getUserCheckbox().getRoot());
      notBeEnabled(this.benutzerPage.getLoeschenCheckbox().getRoot());
    }
  }

  public verifyUserInList(user: AdminUserE2E): void {
    const benutzer: BenutzerListItemE2EComponent = this.getBenutzerItem(user.username);
    contains(benutzer.getUserName(), user.username);
    contains(benutzer.getFullName(), user.vorname);
    contains(benutzer.getFullName(), user.nachname);

    if (user.isUser) contains(benutzer.getRoles(), AlfaRollen.USER);
    if (user.isLoeschen) contains(benutzer.getRoles(), AlfaRollen.LOESCHEN);
    if (user.isPoststelle) contains(benutzer.getRoles(), AlfaRollen.POSTSTELLE);
    if (user.isAdmin) contains(benutzer.getRoles(), AlfaRollen.ADMIN);
  }

  public verifyUserNotInList(userName: string): void {
    notExist(this.getBenutzerItem(userName).getRoot());
  }

  private getBenutzerItem(userName: string): BenutzerListItemE2EComponent {
    return this.benutzerListPage.getItem(userName);
  }

  public verifyEmptyVornameError(): void {
    exist(this.benutzerPage.getVornameTextFeld().getErrorMessage());
    haveText(this.benutzerPage.getVornameTextFeld().getErrorMessage(), E2EBenutzerValidationMessages.VORNAME_EMPTY_ERROR);
  }

  public verifyEmptyNameError(): void {
    exist(this.benutzerPage.getNachnameTextFeld().getErrorMessage());
    haveText(this.benutzerPage.getNachnameTextFeld().getErrorMessage(), E2EBenutzerValidationMessages.NACHNAME_EMPTY_ERROR);
  }

  public verifyUsernameLengthError(): void {
    exist(this.benutzerPage.getBenutzernameTextFeld().getErrorMessage());
    haveText(
      this.benutzerPage.getBenutzernameTextFeld().getErrorMessage(),
      E2EBenutzerValidationMessages.BENUTZERNAME_SIZE_ERROR,
    );
  }

  public verifyUsernameExistsError(): void {
    exist(this.benutzerPage.getBenutzernameTextFeld().getErrorMessage());
    haveText(this.benutzerPage.getBenutzernameTextFeld().getErrorMessage(), E2EBenutzerValidationMessages.BENUTZER_NAME_EXISTS);
  }

  public verifyInvalidEmailError(): void {
    exist(this.benutzerPage.getEmailTextFeld().getErrorMessage());
    haveText(this.benutzerPage.getEmailTextFeld().getErrorMessage(), E2EBenutzerValidationMessages.EMAIL_INVALID_ERROR);
  }

  public verifyEmailExistsError(): void {
    exist(this.benutzerPage.getEmailTextFeld().getErrorMessage());
    haveText(this.benutzerPage.getEmailTextFeld().getErrorMessage(), E2EBenutzerValidationMessages.EMAIL_EXISTS);
  }

  public verifyEmptyRollenError(): void {
    exist(this.benutzerPage.getRollenValidationError());
    contains(this.benutzerPage.getRollenValidationError(), E2EBenutzerValidationMessages.ROLLEN_EMPTY_ERROR);
  }

  public verifyVornameHasNoError(): void {
    haveText(this.benutzerPage.getVornameTextFeld().getErrorMessage(), EMPTY_STRING);
  }

  public verifyLastNameHasNoError(): void {
    haveText(this.benutzerPage.getNachnameTextFeld().getErrorMessage(), EMPTY_STRING);
  }

  public verifyUsernameHasNoError(): void {
    haveText(this.benutzerPage.getBenutzernameTextFeld().getErrorMessage(), EMPTY_STRING);
  }

  public verifyEmailHasNoError(): void {
    haveText(this.benutzerPage.getEmailTextFeld().getErrorMessage(), EMPTY_STRING);
  }

  public verifyRollenHasNoError(): void {
    notContains(this.benutzerPage.getRollenValidationError(), E2EBenutzerValidationMessages.ROLLEN_EMPTY_ERROR);
  }
}
