/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  ConvertForDataTestPipe,
  createStateResource,
  EnumToLabelPipe,
  HasLinkPipe,
  ToResourceUriPipe,
} from '@alfa-client/tech-shared';
import { getElementFromFixture, mock } from '@alfa-client/test-utils';
import { PostfachIconComponent } from '@alfa-client/ui';
import { UserProfileInVorgangListItemContainerComponent } from '@alfa-client/user-profile';
import { UserProfileResource, UserProfileService } from '@alfa-client/user-profile-shared';
import { VorgangHeaderLinkRel, VorgangStatus } from '@alfa-client/vorgang-shared';
import {
  AktenzeichenComponent,
  VorgangNummerComponent,
  VorgangStatusDotComponent,
  VorgangStatusTextComponent,
} from '@alfa-client/vorgang-shared-ui';
import { WiedervorlageListInVorgangListContainerComponent } from '@alfa-client/wiedervorlage';
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { LOCALE_ID } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MAT_DATE_LOCALE } from '@angular/material/core';
import { MatIcon } from '@angular/material/icon';
import { MatIconTestingModule } from '@angular/material/icon/testing';
import { RouterTestingModule } from '@angular/router/testing';
import { getDataTestClassOf, getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createUserProfileResource } from 'libs/user-profile-shared/test/user-profile';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { VorgangArchiveStatusComponent } from './vorgang-archive-status/vorgang-archive-status.component';
import { VorgangBescheidStatusComponent } from './vorgang-bescheid-status/vorgang-bescheid-status.component';
import { VorgangCreatedAtComponent } from './vorgang-created-at/vorgang-created-at.component';
import { VorgangListItemComponent } from './vorgang-list-item.component';
import { VorgangNextFristButtonComponent } from './vorgang-next-frist-button/vorgang-next-frist-button.component';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('VorgangListItemComponent', () => {
  let component: VorgangListItemComponent;
  let fixture: ComponentFixture<VorgangListItemComponent>;

  const userProfileService = mock(UserProfileService);

  const user: string = getDataTestIdOf('vorgang-user-icon');
  const postfachStatus: string = getDataTestClassOf('postfach-icon');
  const bescheidStatus: string = getDataTestIdOf('vorgang-list-item-bescheid-status');
  const archiveStatus: string = getDataTestIdOf('vorgang-list-item-archive-status');

  const userProfile: UserProfileResource = createUserProfileResource();

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [RouterTestingModule, MatIconTestingModule],
      declarations: [
        VorgangListItemComponent,
        MatIcon,
        EnumToLabelPipe,
        ToResourceUriPipe,
        HasLinkPipe,
        ConvertForDataTestPipe,
        MockComponent(AktenzeichenComponent),
        MockComponent(VorgangNummerComponent),
        MockComponent(PostfachIconComponent),
        MockComponent(VorgangStatusDotComponent),
        MockComponent(VorgangStatusTextComponent),
        MockComponent(WiedervorlageListInVorgangListContainerComponent),
        MockComponent(VorgangNextFristButtonComponent),
        MockComponent(UserProfileInVorgangListItemContainerComponent),
        MockComponent(VorgangCreatedAtComponent),
        MockComponent(VorgangBescheidStatusComponent),
        MockComponent(VorgangArchiveStatusComponent),
      ],
      providers: [
        { provide: UserProfileService, useValue: userProfileService },
        { provide: LOCALE_ID, useValue: 'de' },
        { provide: MAT_DATE_LOCALE, useValue: 'de-DE' },
      ],
    });
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(VorgangListItemComponent);
    component = fixture.componentInstance;
    component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG]);
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    describe('user profile', () => {
      beforeEach(() => {
        component.buildAriaLabel = jest.fn();
        component.userProfileService.getAssignedUserProfile = jest.fn().mockReturnValue(of(createStateResource(userProfile)));
      });

      describe('vorgang has no "assigned to" link', () => {
        beforeEach(() => {
          component.vorgang = createVorgangResource();
        });
        it('should not get profile', () => {
          component.ngOnInit();

          expect(component.userProfileService.getAssignedUserProfile).not.toHaveBeenCalled();
        });

        it('should build aria label without user profile resource', () => {
          component.ngOnInit();

          expect(component.buildAriaLabel).toHaveBeenCalledWith();
        });
      });

      describe('vorgang has "assigned to" link', () => {
        beforeEach(() => {
          component.vorgang = createVorgangResource([VorgangHeaderLinkRel.ASSIGNED_TO]);
        });

        it('should get profile', () => {
          component.ngOnInit();

          expect(component.userProfileService.getAssignedUserProfile).toHaveBeenCalled();
        });

        it('should build aria label with user profile resource', () => {
          component.ngOnInit();

          expect(component.buildAriaLabel).toHaveBeenCalledWith(userProfile);
        });
      });
    });
  });

  describe('bearbeiter/user-icon', () => {
    it('should be visible', () => {
      component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG, VorgangHeaderLinkRel.ASSIGN]);
      fixture.detectChanges();

      const userElement = fixture.nativeElement.querySelector(user);

      expect(userElement).toBeInstanceOf(HTMLElement);
    });

    it('should NOT be visible', () => {
      component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG]);
      fixture.detectChanges();

      const userElement = fixture.nativeElement.querySelector(user);

      expect(userElement).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('mail icon', () => {
    beforeEach(() => {
      component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG, VorgangHeaderLinkRel.POSTFACH_MAILS]);
    });

    it('should show mail icon if Vorgang has new Postfachnachricht', () => {
      component.vorgang.hasPostfachNachricht = true;
      fixture.detectChanges();

      const statusElement = getElementFromFixture(fixture, postfachStatus);

      expect(statusElement).toBeInstanceOf(HTMLElement);
    });

    it('should not show mail icon if Vorgang has no new Postfachnachricht', () => {
      component.vorgang.hasPostfachNachricht = false;
      fixture.detectChanges();

      const statusElement = getElementFromFixture(fixture, postfachStatus);

      expect(statusElement).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('Bescheid-Status', () => {
    it('should show bescheid status if Vorgang has antragBewilligt true', () => {
      component.vorgang.antragBewilligt = true;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, bescheidStatus);

      expect(element).toBeInstanceOf(HTMLElement);
    });

    it('should show bescheid status if Vorgang has antragBewilligt false', () => {
      component.vorgang.antragBewilligt = false;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, bescheidStatus);

      expect(element).toBeInstanceOf(HTMLElement);
    });

    it('should not show bescheid status if Vorgang has no antragBewilligt', () => {
      component.vorgang.antragBewilligt = null;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, bescheidStatus);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('Archive-Status', () => {
    it('should show archive status if vorgang has archiving true', () => {
      component.vorgang.archiving = true;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, archiveStatus);

      expect(element).toBeInstanceOf(HTMLElement);
    });

    it('should not show archive status if vorgang has archiving false', () => {
      component.vorgang.archiving = false;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, archiveStatus);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('buildAriaLabel', () => {
    beforeEach(() => {
      component.getWiedervorlageText = jest.fn();
      component.getPostfachNachricht = jest.fn();
    });
    it('should get status', () => {
      component.getStatus = jest.fn();

      component.buildAriaLabel();

      expect(component.getStatus).toHaveBeenCalled();
    });

    it('should get approval text', () => {
      component.getApprovalText = jest.fn();

      component.buildAriaLabel();

      expect(component.getApprovalText).toHaveBeenCalled();
    });

    it('should get user text', () => {
      component.getUserText = jest.fn();

      component.buildAriaLabel(userProfile);

      expect(component.getUserText).toHaveBeenCalledWith(userProfile);
    });

    it('should get Wiedervorlage', () => {
      component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG, VorgangHeaderLinkRel.WIEDERVORLAGEN]);

      component.buildAriaLabel();

      expect(component.getWiedervorlageText).toHaveBeenCalled();
    });

    it('should not get Wiedervorlage if no nextFrist but LinkRel.WIEDERVORLAGEN', () => {
      component.vorgang = {
        ...createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG, VorgangHeaderLinkRel.WIEDERVORLAGEN]),
        nextFrist: null,
      };

      component.buildAriaLabel();

      expect(component.getWiedervorlageText).not.toHaveBeenCalled();
    });

    it('should not get Wiedervorlage if no LinkRel.WIEDERVORLAGEN', () => {
      component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG]);

      component.buildAriaLabel();

      expect(component.getWiedervorlageText).not.toHaveBeenCalled();
    });

    it('should get message text if vorgang has message', () => {
      component.vorgang.hasPostfachNachricht = true;

      component.buildAriaLabel();

      expect(component.getPostfachNachricht).toHaveBeenCalled();
    });

    it('should not get message text if vorgang has no messages', () => {
      component.vorgang.hasPostfachNachricht = false;

      component.buildAriaLabel();

      expect(component.getPostfachNachricht).not.toHaveBeenCalled();
    });
  });

  describe('getWiedervorlageText', () => {
    it('should return text for next resubmission', () => {
      component.vorgang.nextFrist = new Date('07.02.1977');

      const result: string = component.getWiedervorlageText();

      expect(result).toBe(', Nächste Wiedervorlage am 02.07.1977');
    });
  });

  describe('getPostfachNachricht', () => {
    it('should return "contains new messages"', () => {
      component.vorgang.hasNewPostfachNachricht = true;

      const result: string = component.getPostfachNachricht();

      expect(result).toBe(', enthält neue Postfachnachrichten');
    });

    it('should return "contains messages"', () => {
      component.vorgang.hasNewPostfachNachricht = false;

      const result: string = component.getPostfachNachricht();

      expect(result).toBe(', enthält Postfachnachrichten');
    });
  });

  describe('getStatus', () => {
    it('should return status', () => {
      component.vorgang.status = VorgangStatus.NEU;

      const result: string = component.getStatus();

      expect(result).toBe('Neu');
    });
  });

  describe('getApprovalText', () => {
    it('should return empty string', () => {
      component.vorgang.antragBewilligt = undefined;

      const result: string = component.getApprovalText();

      expect(result).toBe('');
    });

    it('should return "approved" text', () => {
      component.vorgang.antragBewilligt = true;

      const result: string = component.getApprovalText();

      expect(result).toBe('bewilligt');
    });

    it('should return "not approved" text', () => {
      component.vorgang.antragBewilligt = false;

      const result: string = component.getApprovalText();

      expect(result).toBe('abgelehnt');
    });
  });

  describe('getUserText', () => {
    it('should return no assigned user text', () => {
      const result: string = component.getUserText(undefined);

      expect(result).toBe('Kein Bearbeiter zugewiesen');
    });

    it('should return assigned user text', () => {
      const result: string = component.getUserText(userProfile);

      expect(result).toBe(`Aktuell zugewiesener Nutzer: ${userProfile.firstName} ${userProfile.lastName}`);
    });
  });
});
